@@ -42,6 +42,7 @@ from .util import EventListener, event_listener
 if TYPE_CHECKING:
     from .network import Network
     from .wallet_db import WalletDB
+    from .simple_config import SimpleConfig
 
 
 TX_HEIGHT_FUTURE = -3
@@ -66,9 +67,10 @@ class AddressSynchronizer(Logger, EventListener):
     synchronizer: Optional['Synchronizer']
     verifier: Optional['SPV']
 
-    def __init__(self, db: 'WalletDB', config):
+    def __init__(self, db: 'WalletDB', config: 'SimpleConfig', *, name: str = None):
         self.db = db
         self.config = config
+        self.name = name
         self.network = None
         Logger.__init__(self)
         # verifier (SPV) and synchronizer are started in start_network
@@ -91,6 +93,9 @@ class AddressSynchronizer(Logger, EventListener):
 
         self.load_and_cleanup()
 
+    def diagnostic_name(self):
+        return self.name or ""
+
     def with_transaction_lock(func):
         def func_wrapper(self: 'AddressSynchronizer', *args, **kwargs):
             with self.transaction_lock:
