@@ -51,7 +51,7 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
 
     isUptodateChanged = pyqtSignal()
     requestStatusChanged = pyqtSignal([str,int], arguments=['key','status'])
-    requestCreateSuccess = pyqtSignal()
+    requestCreateSuccess = pyqtSignal([str], arguments=['key'])
     requestCreateError = pyqtSignal([str,str], arguments=['code','error'])
     invoiceStatusChanged = pyqtSignal([str,int], arguments=['key','status'])
     invoiceCreateSuccess = pyqtSignal()
@@ -62,6 +62,7 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     transactionSigned = pyqtSignal([str], arguments=['txid'])
     #broadcastSucceeded = pyqtSignal([str], arguments=['txid'])
     broadcastFailed = pyqtSignal([str,str,str], arguments=['txid','code','reason'])
+    labelsUpdated = pyqtSignal()
 
     _network_signal = pyqtSignal(str, object)
 
@@ -463,7 +464,7 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
                 fut = asyncio.run_coroutine_threadsafe(coro, self.wallet.network.asyncio_loop)
                 fut.result()
             except Exception as e:
-                self.userNotify.emit(repr(e))
+                self.userNotify.emit(self.wallet, repr(e))
 
         threading.Thread(target=pay_thread).start()
 
@@ -526,7 +527,7 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
 
         assert key is not None
         self._requestModel.add_invoice(self.wallet.get_request(key))
-        self.requestCreateSuccess.emit()
+        self.requestCreateSuccess.emit(key)
 
     @pyqtSlot(str)
     def delete_request(self, key: str):
