@@ -22,7 +22,7 @@ from electrum.gui import messages
 from .util import (MyTreeView, WindowModalDialog, Buttons, OkButton, CancelButton,
                    EnterButton, WaitingDialog, MONOSPACE_FONT, ColorScheme)
 from .amountedit import BTCAmountEdit, FreezableLineEdit
-from .util import read_QIcon
+from .util import read_QIcon, font_height
 
 
 ROLE_CHANNEL_ID = Qt.UserRole
@@ -183,7 +183,7 @@ class ChannelsList(MyTreeView):
         WaitingDialog(self, 'please wait..', task, self.on_request_sent, self.on_failure)
 
     def freeze_channel_for_sending(self, chan, b):
-        if self.lnworker.channel_db or self.lnworker.is_trampoline_peer(chan.node_id):
+        if not self.lnworker.uses_trampoline() or self.lnworker.is_trampoline_peer(chan.node_id):
             chan.set_frozen_for_sending(b)
         else:
             msg = messages.MSG_NON_TRAMPOLINE_CHANNEL_FROZEN_WITHOUT_GOSSIP
@@ -198,7 +198,7 @@ class ChannelsList(MyTreeView):
             channel_id2 = idx2.sibling(idx2.row(), self.Columns.NODE_ALIAS).data(ROLE_CHANNEL_ID)
             chan1 = self.lnworker.channels.get(channel_id1)
             chan2 = self.lnworker.channels.get(channel_id2)
-            if chan1 and chan2 and (self.lnworker.channel_db or chan1.node_id != chan2.node_id):
+            if chan1 and chan2 and (not self.lnworker.uses_trampoline() or chan1.node_id != chan2.node_id):
                 return chan1, chan2
         return None, None
 
@@ -233,7 +233,7 @@ class ChannelsList(MyTreeView):
         if not item:
             return
         channel_id = idx.sibling(idx.row(), self.Columns.NODE_ALIAS).data(ROLE_CHANNEL_ID)
-        chan = self.lnworker.channels.get(channel_id) or self.lnworker.channel_backups[channel_id]
+        chan = self.lnworker.get_channel_by_id(channel_id) or self.lnworker.channel_backups[channel_id]
         menu.addAction(_("Details..."), lambda: self.parent.show_channel_details(chan))
         menu.addSeparator()
         cc = self.add_copy_menu(menu, idx)
@@ -442,9 +442,10 @@ class ChanFeatNoOnchainBackup(ChannelFeature):
 
 
 class ChannelFeatureIcons:
-    ICON_SIZE = QSize(16, 16)
 
     def __init__(self, features: Sequence['ChannelFeature']):
+        size = max(16, font_height())
+        self.icon_size = QSize(size, size)
         self.features = features
 
     @classmethod
@@ -466,17 +467,17 @@ class ChannelFeatureIcons:
         painter.save()
         cur_x = rect.x()
         for feat in self.features:
-            icon_rect = QRect(cur_x, rect.y(), self.ICON_SIZE.width(), self.ICON_SIZE.height())
+            icon_rect = QRect(cur_x, rect.y(), self.icon_size.width(), self.icon_size.height())
             feat.rect = icon_rect
             if rect.contains(icon_rect):  # stay inside parent
-                painter.drawPixmap(icon_rect, feat.icon().pixmap(self.ICON_SIZE))
-            cur_x += self.ICON_SIZE.width() + 1
+                painter.drawPixmap(icon_rect, feat.icon().pixmap(self.icon_size))
+            cur_x += self.icon_size.width() + 1
         painter.restore()
 
     def sizeHint(self, default_size: QSize) -> QSize:
         if not self.features:
             return default_size
-        width = len(self.features) * (self.ICON_SIZE.width() + 1)
+        width = len(self.features) * (self.icon_size.width() + 1)
         return QSize(width, default_size.height())
 
     def show_tooltip(self, evt: QHelpEvent) -> bool:
