@@ -9,10 +9,13 @@ import sys
 import pathlib
 import os
 import platform
-from typing import Optional
+from typing import Optional, TYPE_CHECKING
 import copy
 import subprocess
 
+if TYPE_CHECKING:
+    from .simple_config import SimpleConfig
+
 
 class LogFormatterForFiles(logging.Formatter):
 
@@ -306,17 +309,16 @@ class Logger:
         return ''
 
 
-def configure_logging(config):
+def configure_logging(config: 'SimpleConfig', *, log_to_file: Optional[bool] = None) -> None:
+    from .util import is_android_debug_apk
+
     verbosity = config.get('verbosity')
     verbosity_shortcuts = config.get('verbosity_shortcuts')
     _configure_stderr_logging(verbosity=verbosity, verbosity_shortcuts=verbosity_shortcuts)
 
-    log_to_file = config.get('log_to_file', False)
-    is_android = 'ANDROID_DATA' in os.environ
-    if is_android:
-        from jnius import autoclass
-        build_config = autoclass("org.electrum.electrum.BuildConfig")
-        log_to_file |= bool(build_config.DEBUG)
+    if log_to_file is None:
+        log_to_file = config.get('log_to_file', False)
+        log_to_file |= is_android_debug_apk()
     if log_to_file:
         log_directory = pathlib.Path(config.path) / "logs"
         _configure_file_logging(log_directory)
@@ -349,9 +351,9 @@ def get_logfile_path() -> Optional[pathlib.Path]:
 
 def describe_os_version() -> str:
     if 'ANDROID_DATA' in os.environ:
-        from kivy import utils
-        if utils.platform != "android":
-            return utils.platform
+        #from kivy import utils
+        #if utils.platform != "android":
+        #    return utils.platform
         import jnius
         bv = jnius.autoclass('android.os.Build$VERSION')
         b = jnius.autoclass('android.os.Build')
