@@ -8,7 +8,7 @@ from PyQt5.QtWidgets import (QVBoxLayout, QLabel, QGridLayout, QPushButton,
                              QMessageBox, QFileDialog, QSlider, QTabWidget)
 
 from electrum.gui.qt.util import (WindowModalDialog, WWLabel, Buttons, CancelButton,
-                                  OkButton, CloseButton)
+                                  OkButton, CloseButton, PasswordLineEdit, getOpenFileName)
 from electrum.i18n import _
 from electrum.plugin import hook
 from electrum.util import bh2u
@@ -16,7 +16,7 @@ from electrum.util import bh2u
 from ..hw_wallet.qt import QtHandlerBase, QtPluginBase
 from ..hw_wallet.plugin import only_hook_if_libraries_available
 from .trezor import (TrezorPlugin, TIM_NEW, TIM_RECOVER, TrezorInitSettings,
-                     RECOVERY_TYPE_SCRAMBLED_WORDS, RECOVERY_TYPE_MATRIX)
+                     PASSPHRASE_ON_DEVICE, Capability, BackupType, RecoveryDeviceType)
 
 
 PASSPHRASE_HELP_SHORT =_(
@@ -108,7 +108,7 @@ class MatrixDialog(WindowModalDialog):
 
 class QtHandler(QtHandlerBase):
 
-    pin_signal = pyqtSignal(object)
+    pin_signal = pyqtSignal(object, object)
     matrix_signal = pyqtSignal(object)
     close_matrix_dialog_signal = pyqtSignal()
 
@@ -119,10 +119,11 @@ class QtHandler(QtHandlerBase):
         self.close_matrix_dialog_signal.connect(self._close_matrix_dialog)
         self.pin_matrix_widget_class = pin_matrix_widget_class
         self.matrix_dialog = None
+        self.passphrase_on_device = False
 
-    def get_pin(self, msg):
+    def get_pin(self, msg, *, show_strength=True):
         self.done.clear()
-        self.pin_signal.emit(msg)
+        self.pin_signal.emit(msg, show_strength)
         self.done.wait()
         return self.response
 
@@ -143,11 +144,11 @@ class QtHandler(QtHandlerBase):
     def close_matrix_dialog(self):
         self.close_matrix_dialog_signal.emit()
 
-    def pin_dialog(self, msg):
+    def pin_dialog(self, msg, show_strength):
         # Needed e.g. when resetting a device
         self.clear_dialog()
         dialog = WindowModalDialog(self.top_level_window(), _("Enter PIN"))
-        matrix = self.pin_matrix_widget_class()
+        matrix = self.pin_matrix_widget_class(show_strength)
         vbox = QVBoxLayout()
         vbox.addWidget(QLabel(msg))
         vbox.addWidget(matrix)
@@ -163,15 +164,76 @@ class QtHandler(QtHandlerBase):
         self.matrix_dialog.get_matrix(msg)
         self.done.set()
 
+    def passphrase_dialog(self, msg, confirm):
+        # If confirm is true, require the user to enter the passphrase twice
+        parent = self.top_level_window()
+        d = WindowModalDialog(parent, _('Enter Passphrase'))
+
+        OK_button = OkButton(d, _('Enter Passphrase'))
+        OnDevice_button = QPushButton(_('Enter Passphrase on Device'))
+
+        new_pw = PasswordLineEdit()
+        conf_pw = PasswordLineEdit()
+
+        vbox = QVBoxLayout()
+        label = QLabel(msg + "\n")
+        label.setWordWrap(True)
+
+        grid = QGridLayout()
+        grid.setSpacing(8)
+        grid.setColumnMinimumWidth(0, 150)
+        grid.setColumnMinimumWidth(1, 100)
+        grid.setColumnStretch(1,1)
+
+        vbox.addWidget(label)
+
+        grid.addWidget(QLabel(_('Passphrase:')), 0, 0)
+        grid.addWidget(new_pw, 0, 1)
+
+        if confirm:
+            grid.addWidget(QLabel(_('Confirm Passphrase:')), 1, 0)
+            grid.addWidget(conf_pw, 1, 1)
+
+        vbox.addLayout(grid)
+
+        def enable_OK():
+            if not confirm:
+                ok = True
+            else:
+                ok = new_pw.text() == conf_pw.text()
+            OK_button.setEnabled(ok)
+
+        new_pw.textChanged.connect(enable_OK)
+        conf_pw.textChanged.connect(enable_OK)
+
+        vbox.addWidget(OK_button)
+
+        if self.passphrase_on_device:
+            vbox.addWidget(OnDevice_button)
+
+        d.setLayout(vbox)
+
+        self.passphrase = None
+
+        def ok_clicked():
+            self.passphrase = new_pw.text()
+
+        def on_device_clicked():
+            self.passphrase = PASSPHRASE_ON_DEVICE
+
+        OK_button.clicked.connect(ok_clicked)
+        OnDevice_button.clicked.connect(on_device_clicked)
+        OnDevice_button.clicked.connect(d.accept)
+
+        d.exec_()
+        self.done.set()
+
 
 class QtPlugin(QtPluginBase):
     # Derived classes must provide the following class-static variables:
     #   icon_file
     #   pin_matrix_widget_class
 
-    def create_handler(self, window):
-        return QtHandler(window, self.pin_matrix_widget_class(), self.device)
-
     @only_hook_if_libraries_available
     @hook
     def receive_menu(self, menu, addrs, wallet):
@@ -185,9 +247,13 @@ class QtPlugin(QtPluginBase):
                 menu.addAction(_("Show on {}").format(device_name), show_address)
 
     def show_settings_dialog(self, window, keystore):
-        device_id = self.choose_device(window, keystore)
-        if device_id:
-            SettingsDialog(window, self, keystore, device_id).exec_()
+        def connect():
+            device_id = self.choose_device(window, keystore)
+            return device_id
+        def show_dialog(device_id):
+            if device_id:
+                SettingsDialog(window, self, keystore, device_id).exec_()
+        keystore.thread.add(connect, on_success=show_dialog)
 
     def request_trezor_init_settings(self, wizard, method, device_id):
         vbox = QVBoxLayout()
@@ -199,6 +265,8 @@ class QtPlugin(QtPluginBase):
             raise Exception(_("The device was disconnected."))
         model = client.get_trezor_model()
         fw_version = client.client.version
+        capabilities = client.client.features.capabilities
+        have_shamir = Capability.Shamir in capabilities
 
         # label
         label = QLabel(_("Enter a label to name your device:"))
@@ -209,22 +277,88 @@ class QtPlugin(QtPluginBase):
         hl.addStretch(1)
         vbox.addLayout(hl)
 
+        # Backup type
+        gb_backuptype = QGroupBox()
+        hbox_backuptype = QHBoxLayout()
+        gb_backuptype.setLayout(hbox_backuptype)
+        vbox.addWidget(gb_backuptype)
+        gb_backuptype.setTitle(_('Select backup type:'))
+        bg_backuptype = QButtonGroup()
+
+        rb_single = QRadioButton(gb_backuptype)
+        rb_single.setText(_('Single seed (BIP39)'))
+        bg_backuptype.addButton(rb_single)
+        bg_backuptype.setId(rb_single, BackupType.Bip39)
+        hbox_backuptype.addWidget(rb_single)
+        rb_single.setChecked(True)
+
+        rb_shamir = QRadioButton(gb_backuptype)
+        rb_shamir.setText(_('Shamir'))
+        bg_backuptype.addButton(rb_shamir)
+        bg_backuptype.setId(rb_shamir, BackupType.Slip39_Basic)
+        hbox_backuptype.addWidget(rb_shamir)
+        rb_shamir.setEnabled(Capability.Shamir in capabilities)
+        rb_shamir.setVisible(False)  # visible with "expert settings"
+
+        rb_shamir_groups = QRadioButton(gb_backuptype)
+        rb_shamir_groups.setText(_('Super Shamir'))
+        bg_backuptype.addButton(rb_shamir_groups)
+        bg_backuptype.setId(rb_shamir_groups, BackupType.Slip39_Advanced)
+        hbox_backuptype.addWidget(rb_shamir_groups)
+        rb_shamir_groups.setEnabled(Capability.ShamirGroups in capabilities)
+        rb_shamir_groups.setVisible(False)  # visible with "expert settings"
+
         # word count
-        gb = QGroupBox()
+        word_count_buttons = {}
+
+        gb_numwords = QGroupBox()
         hbox1 = QHBoxLayout()
-        gb.setLayout(hbox1)
-        vbox.addWidget(gb)
-        gb.setTitle(_("Select your seed length:"))
+        gb_numwords.setLayout(hbox1)
+        vbox.addWidget(gb_numwords)
+        gb_numwords.setTitle(_("Select seed/share length:"))
         bg_numwords = QButtonGroup()
-        word_counts = (12, 18, 24)
-        for i, count in enumerate(word_counts):
-            rb = QRadioButton(gb)
+        for count in (12, 18, 20, 24, 33):
+            rb = QRadioButton(gb_numwords)
+            word_count_buttons[count] = rb
             rb.setText(_("{:d} words").format(count))
             bg_numwords.addButton(rb)
-            bg_numwords.setId(rb, i)
+            bg_numwords.setId(rb, count)
             hbox1.addWidget(rb)
             rb.setChecked(True)
 
+        def configure_word_counts():
+            if model == "1":
+                checked_wordcount = 24
+            else:
+                checked_wordcount = 12
+
+            if method == TIM_RECOVER:
+                if have_shamir:
+                    valid_word_counts = (12, 18, 20, 24, 33)
+                else:
+                    valid_word_counts = (12, 18, 24)
+            elif rb_single.isChecked():
+                valid_word_counts = (12, 18, 24)
+                gb_numwords.setTitle(_('Select seed length:'))
+            else:
+                valid_word_counts = (20, 33)
+                checked_wordcount = 20
+                gb_numwords.setTitle(_('Select share length:'))
+
+            word_count_buttons[checked_wordcount].setChecked(True)
+            for c, btn in word_count_buttons.items():
+                btn.setVisible(c in valid_word_counts)
+
+        bg_backuptype.buttonClicked.connect(configure_word_counts)
+        configure_word_counts()
+
+        # set up conditional visibility:
+        # 1. backup_type is only visible when creating new seed
+        gb_backuptype.setVisible(method == TIM_NEW)
+        # 2. word_count is not visible when recovering on TT
+        if method == TIM_RECOVER and model != "1":
+            gb_numwords.setVisible(False)
+
         # PIN
         cb_pin = QCheckBox(_('Enable PIN protection'))
         cb_pin.setChecked(True)
@@ -240,6 +374,8 @@ class QtPlugin(QtPluginBase):
         def show_expert_settings():
             expert_button.setVisible(False)
             expert_widget.setVisible(True)
+            rb_shamir.setVisible(True)
+            rb_shamir_groups.setVisible(True)
         expert_button.clicked.connect(show_expert_settings)
         vbox.addWidget(expert_button)
 
@@ -255,7 +391,7 @@ class QtPlugin(QtPluginBase):
 
         # ask for recovery type (random word order OR matrix)
         bg_rectype = None
-        if method == TIM_RECOVER and not model == 'T':
+        if method == TIM_RECOVER and model == '1':
             gb_rectype = QGroupBox()
             hbox_rectype = QHBoxLayout()
             gb_rectype.setLayout(hbox_rectype)
@@ -266,14 +402,14 @@ class QtPlugin(QtPluginBase):
             rb1 = QRadioButton(gb_rectype)
             rb1.setText(_('Scrambled words'))
             bg_rectype.addButton(rb1)
-            bg_rectype.setId(rb1, RECOVERY_TYPE_SCRAMBLED_WORDS)
+            bg_rectype.setId(rb1, RecoveryDeviceType.ScrambledWords)
             hbox_rectype.addWidget(rb1)
             rb1.setChecked(True)
 
             rb2 = QRadioButton(gb_rectype)
             rb2.setText(_('Matrix'))
             bg_rectype.addButton(rb2)
-            bg_rectype.setId(rb2, RECOVERY_TYPE_MATRIX)
+            bg_rectype.setId(rb2, RecoveryDeviceType.Matrix)
             hbox_rectype.addWidget(rb2)
 
         # no backup
@@ -293,11 +429,12 @@ class QtPlugin(QtPluginBase):
         wizard.exec_layout(vbox, next_enabled=next_enabled)
 
         return TrezorInitSettings(
-            word_count=word_counts[bg_numwords.checkedId()],
+            word_count=bg_numwords.checkedId(),
             label=name.text(),
             pin_enabled=cb_pin.isChecked(),
             passphrase_enabled=cb_phrase.isChecked(),
             recovery_type=bg_rectype.checkedId() if bg_rectype else None,
+            backup_type=bg_backuptype.checkedId(),
             no_backup=cb_no_backup.isChecked() if cb_no_backup else False,
         )
 
@@ -306,6 +443,9 @@ class Plugin(TrezorPlugin, QtPlugin):
     icon_unpaired = "trezor_unpaired.png"
     icon_paired = "trezor.png"
 
+    def create_handler(self, window):
+        return QtHandler(window, self.pin_matrix_widget_class(), self.device)
+
     @classmethod
     def pin_matrix_widget_class(self):
         from trezorlib.qt.pinmatrix import PinMatrixWidget
@@ -402,8 +542,11 @@ class SettingsDialog(WindowModalDialog):
             invoke_client('toggle_passphrase', unpair_after=currently_enabled)
 
         def change_homescreen():
-            dialog = QFileDialog(self, _("Choose Homescreen"))
-            filename, __ = dialog.getOpenFileName()
+            filename = getOpenFileName(
+                parent=self,
+                title=_("Choose Homescreen"),
+                config=config,
+            )
             if not filename:
                 return  # user cancelled
 
