@@ -4,16 +4,12 @@ import random
 
 from typing import Mapping, DefaultDict, Tuple, Optional, Dict, List
 
-from .logging import get_logger, Logger
 from .lnutil import LnFeatures
 from .lnonion import calc_hops_data_for_payment, new_onion_packet
 from .lnrouter import RouteEdge, TrampolineEdge, LNPaymentRoute, is_route_sane_to_use
 from .lnutil import NoPathFound, LNPeerAddr
 from . import constants
 
-
-_logger = get_logger(__name__)
-
 # trampoline nodes are supposed to advertise their fee and cltv in node_update message
 TRAMPOLINE_FEES = [
     {
@@ -74,15 +70,16 @@ TRAMPOLINE_NODES_SIGNET = {
 _TRAMPOLINE_NODES_UNITTESTS = {}  # used in unit tests
 
 def hardcoded_trampoline_nodes() -> Mapping[str, LNPeerAddr]:
-    nodes = {}
-    if constants.net.NET_NAME == "mainnet":
-        nodes.update(TRAMPOLINE_NODES_MAINNET)
+    if _TRAMPOLINE_NODES_UNITTESTS:
+        return _TRAMPOLINE_NODES_UNITTESTS
+    elif constants.net.NET_NAME == "mainnet":
+        return TRAMPOLINE_NODES_MAINNET
     elif constants.net.NET_NAME == "testnet":
-        nodes.update(TRAMPOLINE_NODES_TESTNET)
+        return TRAMPOLINE_NODES_TESTNET
     elif constants.net.NET_NAME == "signet":
-        nodes.update(TRAMPOLINE_NODES_SIGNET)
-    nodes.update(_TRAMPOLINE_NODES_UNITTESTS)
-    return nodes
+        return TRAMPOLINE_NODES_SIGNET
+    else:
+        return {}
 
 def trampolines_by_id():
     return dict([(x.pubkey, x) for x in hardcoded_trampoline_nodes().values()])
@@ -100,9 +97,8 @@ def encode_routing_info(r_tags):
     return result.tobytes()
 
 
-def is_legacy_relay(invoice_features, r_tags) -> Tuple[bool, Optional[bytes]]:
-    """Returns if we deal with a legacy payment and gives back the possible last
-    trampoline pubkey.
+def is_legacy_relay(invoice_features, r_tags) -> Tuple[bool, List[bytes]]:
+    """Returns if we deal with a legacy payment and the list of trampoline pubkeys in the invoice.
     """
     invoice_features = LnFeatures(invoice_features)
     # trampoline-supporting wallets:
@@ -116,7 +112,7 @@ def is_legacy_relay(invoice_features, r_tags) -> Tuple[bool, Optional[bytes]]:
         # Any trampoline node should be able to figure out a path to the receiver and
         # we can use an e2e payment.
         if not r_tags:
-            return False, None
+            return False, []
         else:
             # - We choose one routing hint at random, and
             #   use end-to-end trampoline if that node is a trampoline-forwarder (TF).
@@ -126,12 +122,11 @@ def is_legacy_relay(invoice_features, r_tags) -> Tuple[bool, Optional[bytes]]:
             #   endpoints connected to T1 and T2, and sender only has send-capacity with T1, while
             #   recipient only has recv-capacity with T2.
             singlehop_r_tags = [x for x in r_tags if len(x) == 1]
-            forwarder_pubkey = random.choice(singlehop_r_tags)[0][0]
-            if is_hardcoded_trampoline(forwarder_pubkey):
-                return False, forwarder_pubkey
+            invoice_trampolines = [x[0][0] for x in singlehop_r_tags if is_hardcoded_trampoline(x[0][0])]
+            return False, invoice_trampolines
     # if trampoline receiving is not supported or the forwarder is not known as a trampoline,
     # we send a legacy payment
-    return True, None
+    return True, []
 
 
 def trampoline_policy(
@@ -169,6 +164,18 @@ def extend_trampoline_route(
             node_features=trampoline_features))
 
 
+def choose_second_trampoline(my_trampoline, trampolines, failed_routes):
+    if my_trampoline in trampolines:
+        trampolines.remove(my_trampoline)
+    for r in failed_routes:
+        if len(r) > 2:
+            t2 = bytes.fromhex(r[1])
+            if t2 in trampolines:
+                trampolines.remove(t2)
+    if not trampolines:
+        raise NoPathFound('all routes have failed')
+    return random.choice(trampolines)
+
 def create_trampoline_route(
         *,
         amount_msat: int,
@@ -176,28 +183,28 @@ def create_trampoline_route(
         invoice_pubkey: bytes,
         invoice_features: int,
         my_pubkey: bytes,
-        trampoline_node_id: bytes,  # the first trampoline in the path; which we are directly connected to
+        my_trampoline: bytes,  # the first trampoline in the path; which we are directly connected to
         r_tags,
         trampoline_fee_level: int,
-        use_two_trampolines: bool
+        use_two_trampolines: bool,
+        failed_routes: list,
 ) -> LNPaymentRoute:
     # we decide whether to convert to a legacy payment
-    is_legacy, second_trampoline_pubkey = is_legacy_relay(invoice_features, r_tags)
+    is_legacy, invoice_trampolines = is_legacy_relay(invoice_features, r_tags)
 
     # we build a route of trampoline hops and extend the route list in place
     route = []
+    second_trampoline = None
 
     # our first trampoline hop is decided by the channel we use
-    extend_trampoline_route(route, my_pubkey, trampoline_node_id, trampoline_fee_level)
+    extend_trampoline_route(route, my_pubkey, my_trampoline, trampoline_fee_level)
 
     if is_legacy:
         # we add another different trampoline hop for privacy
         if use_two_trampolines:
             trampolines = trampolines_by_id()
-            del trampolines[trampoline_node_id]
-            second_trampoline_pubkey = random.choice(list(trampolines.keys()))
-            extend_trampoline_route(route, trampoline_node_id, second_trampoline_pubkey, trampoline_fee_level)
-
+            second_trampoline = choose_second_trampoline(my_trampoline, list(trampolines.keys()), failed_routes)
+            extend_trampoline_route(route, my_trampoline, second_trampoline, trampoline_fee_level)
         # the last trampoline onion must contain routing hints for the last trampoline
         # node to find the recipient
         invoice_routing_info = encode_routing_info(r_tags)
@@ -205,19 +212,26 @@ def create_trampoline_route(
         route[-1].invoice_features = invoice_features
         route[-1].outgoing_node_id = invoice_pubkey
     else:
-        if second_trampoline_pubkey and trampoline_node_id != second_trampoline_pubkey:
-            extend_trampoline_route(route, trampoline_node_id, second_trampoline_pubkey, trampoline_fee_level)
+        if invoice_trampolines:
+            if my_trampoline in invoice_trampolines:
+                short_route = [my_trampoline.hex(), invoice_pubkey.hex()]
+                if short_route in failed_routes:
+                    add_trampoline = True
+                else:
+                    add_trampoline = False
+            else:
+                add_trampoline = True
+            if add_trampoline:
+                second_trampoline = choose_second_trampoline(my_trampoline, invoice_trampolines, failed_routes)
+                extend_trampoline_route(route, my_trampoline, second_trampoline, trampoline_fee_level)
 
     # final edge (not part of the route if payment is legacy, but eclair requires an encrypted blob)
     extend_trampoline_route(route, route[-1].end_node, invoice_pubkey, trampoline_fee_level, pay_fees=False)
-
     # check that we can pay amount and fees
     for edge in route[::-1]:
         amount_msat += edge.fee_for_edge(amount_msat)
     if not is_route_sane_to_use(route, amount_msat, min_cltv_expiry):
         raise NoPathFound("We cannot afford to pay the fees.")
-    _logger.info(f'created route with trampoline fee level={trampoline_fee_level}, is legacy: {is_legacy}')
-    _logger.info(f'trampoline hops: {[hop.end_node.hex() for hop in route]}')
     return route
 
 
@@ -255,7 +269,6 @@ def create_trampoline_onion(*, route, amount_msat, final_cltv, total_msat, payme
                 "payment_secret": payment_secret,
                 "total_msat": total_msat
             }
-        _logger.info(f'payload {i} {payload}')
     trampoline_session_key = os.urandom(32)
     trampoline_onion = new_onion_packet(payment_path_pubkeys, trampoline_session_key, hops_data, associated_data=payment_hash, trampoline=True)
     return trampoline_onion, amount_msat, cltv
@@ -275,7 +288,8 @@ def create_trampoline_route_and_onion(
         payment_secret,
         local_height: int,
         trampoline_fee_level: int,
-        use_two_trampolines: bool):
+        use_two_trampolines: bool,
+        failed_routes: list):
     # create route for the trampoline_onion
     trampoline_route = create_trampoline_route(
         amount_msat=amount_msat,
@@ -283,10 +297,11 @@ def create_trampoline_route_and_onion(
         my_pubkey=my_pubkey,
         invoice_pubkey=invoice_pubkey,
         invoice_features=invoice_features,
-        trampoline_node_id=node_id,
+        my_trampoline=node_id,
         r_tags=r_tags,
         trampoline_fee_level=trampoline_fee_level,
-        use_two_trampolines=use_two_trampolines)
+        use_two_trampolines=use_two_trampolines,
+        failed_routes=failed_routes)
     # compute onion and fees
     final_cltv = local_height + min_cltv_expiry
     trampoline_onion, amount_with_fees, bucket_cltv = create_trampoline_onion(
@@ -301,4 +316,4 @@ def create_trampoline_route_and_onion(
     # trampoline fee for this very trampoline
     trampoline_fee = trampoline_route[0].fee_for_edge(amount_with_fees)
     amount_with_fees += trampoline_fee
-    return trampoline_onion, amount_with_fees, bucket_cltv_delta
+    return trampoline_route, trampoline_onion, amount_with_fees, bucket_cltv_delta
